import {width, height, margin, colors} from "./constants.js";
import {legend} from "./country-legend.js";


// Load the combined dataset
Promise.all([
    d3.csv("schengen_data.csv"), // Schengen data
    d3.csv("world-data-2023.csv")  // Country data
]).then(([schengenData1, countryData]) => {
    // Map country data by country name for quick lookup
    const countryMap = {};
    countryData.forEach(d => {
        countryMap[d.Country] = {
            gdp: +d.GDP.replace(/[$,]/g, ''), // Parse GDP
            population: +d.Population.replace(/[$,]/g, '') // Parse population
        };
    });

    // Merge GDP and population data into Schengen dataset
    schengenData1.forEach(d => {
        const countryInfo = countryMap[d["Schengen State"]];
        d.GDP = countryInfo ? countryInfo.gdp : 0;
        d.Population = countryInfo ? countryInfo.population : 0;
    });

    const schengenData = Array.from(d3.rollup(
        schengenData1,
        v => ({
            GDP: v[0].GDP,
            Population: v[0].Population,
            "Visas Issued": d3.sum(v, d => d["Total  uniform visas issued (including MEV)"]),
            "Visas Applied for": d3.sum(v, d => d["Total ATVs and uniform visas applied for"])
        }),
        d => d["Schengen State"]
    )).map(([key, value]) => ({
        "Schengen State": key,
        GDP: value.GDP,
        Population: value.Population,
        "Visas Issued": value["Visas Issued"],
        "Visas Applied for": value["Visas Applied for"]
    }));

    console.log(schengenData);


    const xScale = d3.scaleLog()
        .domain([d3.min(schengenData, d => d.GDP), d3.max(schengenData, d => d.GDP)])
        .range([0, width-margin.left-margin.right-130]);

    const yScale = d3.scaleLog()
        .domain([d3.min(schengenData, d => d.Population), d3.max(schengenData, d => d.Population)])
        .range([height-margin.top-margin.bottom-110, 0]);

    const sizeScale = d3.scaleSqrt()
        .domain([0, d3.max(schengenData, d => d["Visas Issued"] || 1)])
        .range([8, 60]);

    const opacityScale = d3.scaleSqrt()
        .domain([0, d3.max(schengenData, d => d["Visas Applied for"] || 1)])
        .range([0.1, 1]);

    //const colorScale = d3.scaleOrdinal(d3.schemeCategory10);
    const colorScale = d3
        .scaleOrdinal()
        .domain(schengenData)
        .range(colors)


    // Append SVG container
    const svg = d3.select("#bubble-container")
        .append("svg")
        .attr("width", width)
        .attr("height", height)
        .append("g")
        .attr("transform", `translate(${margin.left+80},${margin.top+50})`);

    // Add X axis
    svg.append("g")
        .attr("transform", `translate(0,${height-margin.top-margin.bottom-110})`)
        .call(d3.axisBottom(xScale).ticks(10, ".1s"))
        .append("text")
        .attr("class", "axis-label")
        .attr("transform", `translate(${(width-margin.left-margin.right-150)/2},50)`)
        .text("GDP")
        .style("fill", "black");

    // Add Y axis
    svg.append("g")
        .call(d3.axisLeft(yScale))
        .append("text")
        .attr("class", "axis-label")
        //.attr("x", -height / 2)
        .attr("y", -50)
        .attr("transform", `rotate(-90) translate(${-(height-margin.top-margin.bottom-120)/2},-10)`)
        .text("Population")
        .style("fill", "black");

    // Draw bubbles
    const tooltip = d3.select("body")
        .append("div")
        .attr("class", "tooltip")
        .style("visibility", "hidden");

    svg.selectAll("circle")
        .data(schengenData)
        .enter()
        .append("circle")
        .attr("cx", d => xScale(d.GDP || 1e10))
        .attr("cy", d => yScale(d.Population || 1))
        .attr("r", d => Math.round(sizeScale(d["Visas Issued"] || 1)))
        .style("opacity", d => opacityScale(d["Visas Applied for"] || 1))
        .attr("fill", d => colorScale(d["Schengen State"]))
        .on("mouseover", (event, d) => {
            tooltip.style("visibility", "visible")
                .html(`
                            <strong>${d["Schengen State"]}</strong><br>
                            GDP: $${d.GDP.toLocaleString()}<br>
                            Population: ${d.Population.toLocaleString()}<br>
                            Total Visas Issued: ${d["Visas Issued"].toLocaleString()}
                        `)
                .style("left", `${event.pageX + 10}px`)
                .style("top", `${event.pageY - 20}px`);
        })
        .on("mouseout", () => {
            tooltip.style("visibility", "hidden");
        });

    const allStates = new Set(schengenData.map(d => d["Schengen State"]));
    legend(svg, allStates, colorScale)
});
