import {width, height, margin} from "./constants.js";

// Append SVG
const svg = d3.select("#parallel-coordinates-container")
    .append("svg")
    .attr("width", width)
    .attr("height", height)
    .append("g")
    .attr("transform", `translate(${margin.left+margin.right-10},${margin.top+margin.bottom})`);

const loadingIndicator = d3.select("#loading");

// Show loading indicator
function showLoading() {
    loadingIndicator.style("display", "block");
}

// Hide loading indicator
function hideLoading() {
    loadingIndicator.style("display", "none");
}

// Load data
showLoading();
d3.csv("schengen_data.csv").then((data) => {
    // Parse data
    data.forEach(d => {
        for (const key in d) {
            if (!isNaN(+d[key])) {
                d[key] = +d[key];
            }
        }
    });

    // Define dimensions and scales
    const dimensions = [
        "Airport transit visas (ATVs) applied for",
        "ATVs issued (including multiple)",
        "Not issued rate for ATVs",
        "Uniform visas applied for",
        "Total  uniform visas issued (including MEV)",
        "Not issued rate for uniform visas"
    ];

    const yScales = {};
    dimensions.forEach(dim => {
        yScales[dim] = d3.scaleLinear()
            .domain(d3.extent(data, d => d[dim]))
            .range([height-margin.top-margin.bottom-30, 0]);
    });

    const xScale = d3.scalePoint()
        .domain(dimensions)
        .range([0, width-margin.left-margin.right]);

    // Line generator
    const lineGenerator = d3.line()
        .defined(d => !isNaN(d[1]))
        .x(d => xScale(d[0]))
        .y(d => yScales[d[0]](d[1]));

    // Draw axes with labels
    dimensions.forEach(dim => {
        svg.append("g")
            .attr("transform", `translate(${xScale(dim)},0)`)
            .call(d3.axisLeft(yScales[dim]));

        // Add axis label
        svg.append("text")
            .attr("transform", `translate(${xScale(dim)},${height-margin.left-margin.right})`)
            .style("text-anchor", "middle")
            .style("font-size", "12px")
            .text(dim);
    });


    // Group data by state
    const groupedData = d3.group(data, d => d["Schengen State"]);
    const states = Array.from(groupedData.keys());

    // Populate dropdown
    const dropdown = d3.select("#state-dropdown");
    dropdown.selectAll("option")
        .data(states)
        .enter()
        .append("option")
        .attr("value", d => d)
        .text(d => d);

    // Update plot function
    function updatePlot(state) {
        const filteredData = state ? groupedData.get(state) : data;

        if (!filteredData) {
            console.warn(`No data available for state: ${state}`);
            svg.selectAll(".data-line").remove();
            return;
        }

        const paths = svg.selectAll(".data-line")
            .data(filteredData);

        // Enter and update paths
        paths.enter()
            .append("path")
            .attr("class", "data-line")
            .merge(paths)
            .attr("d", d => lineGenerator(dimensions.map(dim => [dim, d[dim]])))
            .style("stroke", "steelblue")
            .style("opacity", 0.1)
            .attr("fill", "none");


        // Remove old paths
        paths.exit().remove();
    }

    // Initialize with full dataset
    updatePlot(null);

    // Dropdown change event
    dropdown.on("change", function() {
        const selectedState = d3.select(this).property("value");
        updatePlot(selectedState);
    });

    // Add reset button functionality
    d3.select("#reset-button")
        .on("click", () => {
            dropdown.property("value", "");
            updatePlot(null);
        });

    // Hide loading indicator after rendering
    hideLoading();
}).catch(err => {
    console.error("Error loading data: ", err);
    hideLoading();
});